#! /bin/make
##############################################
# common make definitions for samples
#===========================================
#        == DO NOT CHANGE !!! ==
##############################################
TARGET:=$(shell basename `pwd`)
OSTYPE := $(shell uname | cut -d _ -f 1 | tr [:upper:] [:lower:])
HOSTTYPE := $(shell uname -m)

#-------------------------------------------
WORK=$(ROOT_PATH)/..
COMMON_PATH=$(WORK)/common
IMPACT_PATH=$(COMMON_PATH)/Impact
SYSTEM_INC_PATH=$(COMMON_PATH)/System
SYSTEM_PATH=$(SYSTEM_INC_PATH)/linux

#-------------------------------------------
include $(SYSTEM_PATH)/Makefile.all.inc

#-------------------------------------------
SOURCES= $(MODULE_NAME).cpp
OBJECTS= $(MODULE_NAME).o
HEADERS= *.h

ifeq ($(BINARY_NAME),)
	BINARY_NAME=$(MODULE_NAME)
endif

#-------------------------------------------
#-------------------------------------------
## ATENTION!!
# Feature in the glibc!
# *** glibc detected *** free(): invalid pointer: 0xXXXXXXXX ***
# You can control it on a per-run basis by setting the MALLOC_CHECK_ environment variable.
# If you explicitly set this variable it performs even more strict tests than normal.
# 0 -- Do not generate an error message, and do not kill the program
# 1 -- Generate an error message, but do not kill the program
# 2 -- Do not generate an error message, but kill the program
# 3 -- Generate an error message and kill the program
# need you need to: export MALLOC_CHECK_=1
#-------------------------------------------
#-------------------------------------------

#-------------------------------------------
############################################
# Entries for the different target-types
#===========================================
############################################
# libs
#===========================================
# this is just "/usr/lib" when not cross compiling
EXPAT_PATH=-L$(CROSS_USER_LIB_PATH)

#-------------------------------------------
IMPACTLIB_PATH=$(mvIMPACT)/lib/$(IPL_LIB_TRGT)

#-------------------------------------------
LOCDEFINES=						\

#-------------------------------------------
INCLFILES=						\
	$(GLOBINCFILES)				\
	-I$(WORK)					\
	-I$(SYSTEM_INC_PATH)		\
	-I$(IMPACT_PATH)/linux		\
	-I$(IMPACT_PATH)			\
	$(USR_INCLS)				\
	-I..

#---------------------------------------------
ifeq ($(HOSTTYPE),x86_64)
	ifeq ($(TARGET),x86)
		# building 32 bit on 64 bit system
		COMP_OPTIONS_32BIT=-m32
	endif
endif

#-------------------------------------------
CPPFLAGS=						\
	$(CPPFLAGS_2)				\
	$(COMP_OPTIONS_32BIT)		\
    $(LOCDEFINES)				\
    $(INCLFILES)

#-------------------------------------------
LDFLAGS = 						\
	$(LDOPT_1)					\
	$(ARCH_LDFLAGS)				\
	$(MV_LIBS)					\
	$(USR_LIBS)					\
	-lm							\
	-lpthread					\
	$(EXPAT_PATH)				\
	-lexpat	

#e.g.
# MV_LIBS = 					\
#		-lmvPropHandling		\
#		-lmvDeviceManager		\
#		-lSampleDevice

#-----------------------------------------
# You may need to adapt this path for your system.
# By default uses "which" except on a mvBL or if a
# special purpose-built wxWidgets exists.
#-----------------------------------------
ifeq ($(DEST),ppc603e)
	WX_CONF_PATH=$(GCCPATH)/bin/
	WX_CONF_TOOL=$(WX_CONF_PATH)wx-config
	ADD_LIBS=							\
		-lz -lX11 -lXext -lpng -ljpeg
else
	# check for specially-built static wxWidgets
	WX_SPECIAL_CONF_TOOL:=$(shell ls /opt/wx/2.6/$(DEST)/bin/wx-config 2>/dev/null)
	ifneq ($(WX_SPECIAL_CONF_TOOL),)
		WX_CONF_TOOL:=$(WX_SPECIAL_CONF_TOOL)
	else
		# use the system's wxWidgets
		WX_CONF_TOOL:=$(shell which wx-config-2.6 2>/dev/null)
		ifeq ($(WX_CONF_TOOL),)
			WX_CONF_TOOL:=$(shell which wx-config 2>/dev/null)
		endif
	endif
	ADD_LIBS=
endif

############################################
#===========================================
.PHONY: run strip dbg clean_mod new pack all $(MODULE_NAME) $(MODULE_NAME) extract_ar_objects ipk

all:	$(MODULE_NAME)

#-------------------------------------------
ifeq ($(HAVE_GCC),yes)
# check if the compiler is installed
#-------------------------------------------

#-------------------------------------------
$(MODULE_NAME): $(OBJECTS) $(OOBJ) extract_ar_objects
	$(CXX) $(CPPFLAGS) -o $(BINARY_NAME) $(OBJECTS) $(OOBJ) $(LLOCLIB) $(LDFLAGS) $(USER_AR_OBJ_LIST) -ldl $(PROFOPT)

#-------------------------------------------
strip: all
	ls -lh $(BINARY_NAME)
	$(TOOLS)strip $(BINARY_NAME)
	ls -lh $(BINARY_NAME)

#-------
else
#-------
$(MODULE_NAME) strip:
	@echo "*** WARNING: No compiler: >>$(CC_TOOL_PATH)gcc<< found!!"

endif


#-------------------------------------------
clean_mod:
	rm -f $(OBJECTS) $(OOBJ) $(MODULE_NAME) $(BINARY_NAME)

#-------------------------------------------

ifeq ($(OSTYPE),cygwin)
.cpp.o:
	@ echo "Compiling $*.cpp"
	-$(CXX) -c $(CPPFLAGS) $(INCLUDES) -o $@ $< 2>gccErrors.dat
	@ gccerr gccErrors.dat
endif

#-------------------------------------------
#-------------------------------------------

