#! /bin/make
##############################################
# common make definitions
# supported CPUs : x86, ppc603e, arm and x86_64
#===========================================
#        == DO NOT CHANGE !!! ==
#===========================================

GPV_MAJOR=1
GPV_MINOR=7
GPV_BUILD=1
GPV_BUILD2=45
GLOBAL_PACKAGE_VERSION=$(GPV_MAJOR).$(GPV_MINOR).$(GPV_BUILD)

#-------------------------------------------------
# SYSTEM_PATH will be set in the Makefile.inc
# of each library!
#-------------------------------------------------
include $(SYSTEM_PATH)/config.inc

#-------------------------------------------------
#-------------------------------------------------
# for CYGWIN user
#---------------------------------------------
ifeq ($(OSTYPE),cygwin)
	OTYPEDEF=-D__cygwin__
	TARGET=ppc603e
	TARGET_SYST=$(OSTYPE)
else
	OTYPEDEF=
	TARGET_SYST=$(TARGET)
endif

#---------------------------------------------
GCCPATH=$(GCC_$(HOSTTYPE)X$(TARGET_SYST))


#---------------------------------------------
# when not cross compiling just set these to the standard directories
CROSS_USER_INC_PATH=/usr/include
CROSS_USER_LIB_PATH=$(shell if [ -h /usr/lib ]; then echo "/usr/lib"; else if [ -e /usr/lib64 ]; then echo "/usr/lib64"; else echo "/usr/lib"; fi; fi)
DISTCC=
CCACHE=

ifeq ($(TARGET),ppc603e)
	CROSS_USER_INC_PATH=$(GCCPATH)/usr/include
	CROSS_USER_LIB_PATH=$(GCCPATH)/usr/lib

	# for compatibility with older Makefiles
	PPC_USER_INC_PATH=$(CROSS_USER_INC_PATH)
	PPC_USER_LIB_PATH=$(CROSS_USER_LIB_PATH)

	CROSS_USR_INC=-I$(CROSS_USER_INC_PATH)

	CROSS_PREFIX=powerpc-linux-

	DISTCC=$(shell which distcc 2>/dev/null)
	CCACHE=$(shell which ccache 2>/dev/null)

else
	ifeq ($(TARGET),arm)
# alternative toolchains
 		CROSS_USER_INC_PATH=$(GCCPATH)/arm-linux/include
 		CROSS_USER_LIB_PATH=$(GCCPATH)/arm-linux/lib
#
# Gentoo toolchain
#		CROSS_USER_INC_PATH=/usr/arm-any-linux-gnu/usr/include/
#		CROSS_USER_LIB_PATH=/usr/arm-any-linux-gnu/usr/lib/

		CROSS_USR_INC=-I$(CROSS_USER_INC_PATH)

# alternative toolchains
		CROSS_PREFIX=arm-linux-
# Gentoo toolchain
#		CROSS_PREFIX=arm-unknown-linux-gnu-
	else
		ifeq ($(TARGET),x86)
			ifeq ($(HOSTTYPE),x86_64)
				# building for 32 bit on 64 bit system - use 64 bit compiler with -m32 option
				CROSS_USER_INC_PATH=/usr/include
				CROSS_USER_LIB_PATH=/usr/lib32
			endif
		endif
	endif
endif

#---------------------------------------------
ifeq ($(CCACHE),)
	# ccache not found, just use distcc if found
	SPEED_PREFIX = $(DISTCC)

else
	# ccache found, use it
	ifneq ($(DISTCC),)
		# use distcc with ccache if found
		export CCACHE_PREFIX = distcc
	endif

	SPEED_PREFIX = ccache

endif

#-------------------------------------------------
#-------------------------------------------------
GLOBINCFILES=								\
	$(PPC_USR_INC)							\

#---------------------------------------------
#===========================================
# OPT
# ---
# To get debug-info set OPT=-g; with OPT=-O2
# you get the sources optimized; if OPT=-pg
# -a there will be generated a profiling
# file (bb.out).
#
#===========================================
#---------------------------------------------
ifeq ($(OPT),)
	OPT=-O2
endif

#---------------------------------------------
#===========================================
# compile level options
# -fPIC	 needed for SO
#===========================================
COMP_OPTIONS=				\
	-Wall -W -fPIC -D_REENTRANT -D_GNU_SOURCE	\
	-D_MAJOR_VERSION=$(GPV_MAJOR)				\
	-D_MINOR_VERSION=$(GPV_MINOR)				\
	-D_BUILD_VERSION=$(GPV_BUILD)				\
	-D_BUILD2_VERSION=$(GPV_BUILD2)				\

#---------------------------------------------
CPPFLAGS_2=				\
	$(OTYPEDEF)			\
	$(INCFILES)			\
   	$(OPT)				\
	$(COMP_OPTIONS)		\
	$(CPPFLAGS_1)

#-------------------------------------------------
#-------------------------------------------------
CC_TOOL_PATH=$(GCCPATH)/bin/$(CROSS_PREFIX)
TOOLS= $(SPEED_PREFIX) $(CC_TOOL_PATH)

#---------------------------------------------
#===========================================
# modifier for built-in rules
#===========================================
	CC  = $(TOOLS)gcc
	CXX = $(TOOLS)g++
#AR	= $(CC_TOOL_PATH)ar
AR	= ar

HAVE_GCC := $(shell if [ -x $(CC_TOOL_PATH)gcc ]; then echo "yes"; else echo "no"; fi )

#---------------------------------------------
#---------------------------------------------
.PHONY: new default info build $(TARGET)
#===========================================
default:	all

#-------------------------------------------
build: clean_mod all

#-------------------------------------------
new: clean_mod strip

#-------------------------------------------
$(TARGET_SYST):
#	mkdir -p $(TARGET)

#-------------------------------------------
$(TARGET_SYST)/%.o:	%.c
	$(CC) -c $(CPPFLAGS)	-o $@ $<

#-------------------------------------------
$(TARGET_SYST)/%.o:	%.cpp
	$(CXX)	-c $(CPPFLAGS) -o $@ $<

#-------------------------------------------
info:
	@echo "==================================================="
	@echo -n "PWD         =" ; pwd
	@echo "OTSYPE         =" $(OSTYPE)
	@echo "HOSTTYPE       =" $(HOSTTYPE)
	@echo "TARGET         =" $(TARGET)
	@echo "ROOT_PATH      =" $(ROOT_PATH)
	@echo "SYSTEM_PATH    =" $(SYSTEM_PATH)
	@echo "TOOLS          =" $(TOOLS)
	@echo "INCFILES       =" $(INCFILES)
	@echo "CC             =" $(CC)
	@echo "CXX            =" $(CXX)
	@echo "CPPFLAGS       =" $(CPPFLAGS)
	@echo "LDOPT          =" $(LDOPT)
	@echo "LDFLAGS        =" $(LDFLAGS)
	@echo "TARGET_SYST    =" $(TARGET_SYST)
	@echo "GCCPATH        =" $(GCCPATH)
	@echo "CROSS_USER_LIB_PATH" = $(CROSS_USER_LIB_PATH)
	@echo "CROSS_USER_INC_PATH" = $(CROSS_USER_INC_PATH)
	@echo "DISTCC         =" $(DISTCC)
	@echo "CCACHE         =" $(CCACHE)
	@echo "CCACHE_PREFIX  =" $(CCACHE_PREFIX)
	@echo "CC_TOOL_PATH   =" $(CC_TOOL_PATH)
	@echo "WX_CONF_PATH   =" $(WX_CONF_PATH)
	@echo "WX_CONF_TOOL   =" $(WX_CONF_TOOL)
	@echo "==================================================="

#-------------------------------------------------
#-------------------------------------------------
