#! /bin/make
############################################
# Makefile.inc for generating libraries.
# 08.08.2005 td
#===========================================
#        == DO NOT CHANGE !!! ==
############################################
TARGET:=$(shell basename `pwd`)
OSTYPE := $(shell uname | cut -d _ -f 1 | tr [:upper:] [:lower:])
HOSTTYPE := $(shell uname -m)

#-------------------------------------------
#-------------------------------------------
USER_AR_OBJ_LIST=

#-------------------------------------------
include $(SYSTEM_PATH)/Makefile.all.inc

#-------------------------------------------
ifeq ($(TARGET),ppc603e)
	ifeq ($(OSTYPE),cygwin)
		LIB_OUT=$(OSTYPE)
	else
		LIB_OUT=$(TARGET)
	endif
else
	ifeq ($(TARGET),arm)
		LIB_OUT=$(TARGET)
	else
		ifeq ($(TARGET),x86_64)
			LIB_OUT=$(TARGET)
		else
			LIB_OUT=x86
		endif
	endif
endif

LIBPATH=$(WORK)/lib/$(LIB_OUT)

#-------------------------------------------
STATIC_LIB=$(LIB_NAME).a

DYNAMIC_LIB=$(LIB_NAME).so
VERSION=$(MAJOR).$(MINOR).$(BUILD)

#-------------------------------------------
INCLFILES=						\
	$(GLOBINCFILES)				\
	-I$(SYSTEM_INC_PATH)		\
	-I$(WORK)/common			\
	-I$(ROOT_PATH)/Include		\
	-I$(WORK)					\
	-I..

#-------------------------------------------
LOCDEFINES=							\

#---------------------------------------------
# building 32 bit on 64 bit system
ifeq ($(HOSTTYPE),x86_64)
	ifeq ($(TARGET),x86)
		COMP_OPTIONS_32BIT=-m32
	endif
endif

#-------------------------------------------
CPPFLAGS=							\
	$(CPPFLAGS_2)					\
	$(COMP_OPTIONS_32BIT)			\
    $(LOCDEFINES)					\
    $(INCLFILES)					\

#-------------------------------------------
LDOPT = 							\
	$(LDOPT_1)						\
	$(COMP_OPTIONS_32BIT)			\

#		--verbose

#===========================================
.PHONY: all strip clean_stat clean_dyn $(STATIC_LIB) $(DYNAMIC_LIB).$(MAJOR) ltarget extract_ar_objects

#-------------------------------------------
all: $(STATIC_LIB) $(DYNAMIC_LIB).$(MAJOR)

#-------------------------------------------
ifeq ($(HAVE_GCC),yes)
# check if the compiler is installed
#-------------------------------------------

ifeq ($(DO_STATIC_LIBS),y)
$(STATIC_LIB):	ltarget extract_ar_objects $(TARGET) $(OBJECTS) $(LIBPATH)
	$(AR) rcs $(LIBPATH)/$(STATIC_LIB) $(OBJECTS) $(EXTERN_OBJ) $(USER_AR_OBJ_LIST)

#-------
else
#-------
$(STATIC_LIB):
	@echo "*** WARNING: No static libs made!! Set 'DO_STATIC_LIBS' to 'y'!!"

endif

#-------------------------------------------
ifeq ($(DO_DYNAMIC_LIBS),y)
$(DYNAMIC_LIB).$(MAJOR):	ltarget $(TARGET) $(OBJECTS) $(LIBPATH)
	$(CTYPE) -shared -Wl,-soname,$(DYNAMIC_LIB)											\
		-o $(LIBPATH)/$(DYNAMIC_LIB).$(VERSION) $(OBJECTS) $(EXTERN_OBJ) $(USE_LN_LIBS) -lc $(LDOPT)
	@( 	cd $(LIBPATH);																	\
		rm -f $(DYNAMIC_LIB) $(DYNAMIC_LIB).$(MAJOR) ;									\
		ln -s $(DYNAMIC_LIB).$(VERSION) $(DYNAMIC_LIB).$(MAJOR) ;						\
		ln -s $(DYNAMIC_LIB).$(MAJOR) $(DYNAMIC_LIB) ;									\
	)

strip: all
	ls -lh $(LIBPATH)/$(DYNAMIC_LIB).$(VERSION)
	$(TOOLS)strip $(LIBPATH)/$(DYNAMIC_LIB).$(VERSION)
	ls -lh $(LIBPATH)/$(DYNAMIC_LIB).$(VERSION)
#-------
else
#-------
$(DYNAMIC_LIB).$(MAJOR):
	@echo "*** WARNING: No dynamic libs made!! Set 'DO_DYNAMIC_LIBS' to 'y'!!"

strip: all

endif

#-------
else
# if the compiler is not installed print the message!
#-------
$(STATIC_LIB) $(DYNAMIC_LIB).$(MAJOR) strip:
	@echo "*** WARNING: No compiler: >>$(CC_TOOL_PATH)gcc<< found!!"

#-------------------------------------------
endif
#-------------------------------------------

#-------------------------------------------
clean_mod: clean_stat clean_dyn

#-------------------------------------------
clean_stat:
	@rm -f $(OBJECTS) $(LIBPATH)/$(STATIC_LIB)

#-------------------------------------------
clean_dyn:
	@rm -f $(OBJECTS) $(LIBPATH)/$(DYNAMIC_LIB)*

#-------------------------------------------
ltarget:
	@mkdir -p $(LIBPATH)


#-------------------------------------------
linfo:
	@echo "WORK          =" $(WORK)
	@echo "TOOLS         =" $(TOOLS)
	@echo "LIBPATH       =" $(LIBPATH)
	@echo "LIB_NAME      =" $(LIB_NAME)
	@echo "STATIC_LIB    =" $(STATIC_LIB)
	@echo "DYNAMIC_LIB   =" $(DYNAMIC_LIB)
	@echo "CTYPE         =" $(CTYPE)
	@echo "INCLFILES     =" $(INCLFILES)
	@echo "VERSION       =" $(VERSION)
	@echo "WX_CONF_PATH  =" $(WX_CONF_PATH)
	@echo "WX_CONF_TOOL  =" $(WX_CONF_TOOL)
	@echo "WXLIBFLAGS    =" $(WXLIBFLAGS)
	@echo "WXCPPFLAGS    =" $(WXCPPFLAGS)


#-------------------------------------------
ifeq ($(USER_AR_LIB_LIST),)
extract_ar_objects:

else
extract_ar_objects: extract_ar
#USER_AR_OBJ_LIST=$(shell xargs ls tmp/*)
USER_AR_OBJ_LIST=$(shell find tmp/ -name "*.o" | xargs ls )

extract_ar:
	@set -x
	@mkdir -p tmp
	@cd tmp
	@for i in $(USER_AR_LIB_LIST);				\
		do 										\
			cd tmp;								\
			if [ -f ../$$i ];					\
			then								\
				$(TOOLS)$(AR) x ../$$i;			\
			else								\
				if [ -f $$i ];					\
				then							\
					$(TOOLS)$(AR) x $$i;		\
				else							\
					echo "**** no archive: '$$i' found!!";	\
				fi;								\
			fi;									\
			rm -f *_*_.SYMDEF;					\
			cd ..;								\
		done;

endif

#-------------------------------------------
#-------------------------------------------
# make ipkg package
#-------------------------------------------
IPKGPACK_NAME=mvimpact-acquire
IPKG_PATH=$(ROOT_PATH)/ipkg
IPKG_LIB_PATH=$(IPKG_PATH)/usr/lib
LIB_LIST=libmvBlueLYNX.so libmvDeviceManager.so libmvPropHandling.so

IPK_VERSION=$(MAJOR).$(MINOR).$(BUILD).$(GPV_BUILD2)
IPKG_PACKAGE=$(IPKGPACK_NAME)_$(IPK_VERSION)_$(TARGET).ipk

#-------------------------------------------
ipk: $(IPKG_PACKAGE)

#-------------------------------------------
ipkglink: stripipkglibs
	@(   																	\
	    if [ -d $(IPKG_PATH) ]; 											\
	    then    															\
			rm -rf $(IPKG_PATH)/usr;										\
			mkdir -p $(IPKG_LIB_PATH);										\
			echo "Copy files....";											\
			cd $(LIBPATH);													\
			for libfiles in $(LIB_LIST);									\
			do																\
				LIBFILES=`ls $$libfiles*`;									\
				for libfile in $$LIBFILES;									\
				do															\
					if [ ! -L $$libfile ];									\
					then													\
						cd -;												\
						cp -v $(LIBPATH)/$$libfile $(IPKG_LIB_PATH)/. ;		\
						cd $(LIBPATH);										\
					fi;														\
				done;														\
			done;															\
			cd -;															\
	    fi  																\
	)

#-------------------------------------------
stripipkglibs:
	@(											\
		echo "Strip files....";					\
		cd $(LIBPATH);							\
		for libfiles in $(LIB_LIST);			\
		do										\
			LIBFILES=`ls $$libfiles*`;			\
			for libfile in $$LIBFILES;			\
			do									\
				if [ ! -L $$libfile ];			\
				then							\
					ls -lh $$libfile;			\
					$(TOOLS)strip $$libfile;	\
					ls -lh $$libfile;			\
				fi;								\
			done;								\
		done;									\
		cd -;									\
	)

#-------------------------------------------
$(IPKG_PACKAGE): ipkglink
	@(   \
	    if [ -d $(IPKG_PATH) ]; 									\
	    then														\
			cd $(ROOT_PATH);										\
			rm -if $(IPKG_PACKAGE);									\
			sed -e "s/^\(Architecture:[\t ]*\)[a-zA-Z0-9.+\-]*/\1$(TARGET)/" < ipkg/CONTROL/control > ipk1.tmp; \
	        sed -e "s/^\(Version:[\t ]*\)[a-zA-Z0-9.+]*/\1$(IPK_VERSION)/" < ipk1.tmp > ipk.tmp; \
			rm ipk1.tmp;    										\
	        mv -v ipk.tmp ipkg/CONTROL/control; 					\
			echo "IPKG_PACKAGE: >$(IPKG_PACKAGE)<";					\
			ipkg-build ipkg;    									\
			cd -;													\
	    fi  														\
	)

#===========================================


